@@ -40,8 +40,8 @@ function(crn, fname, header=NULL, append=FALSE)
     lead.invs = header$lead.invs[1]
     comp.date = header$comp.date[1]
 
-    site.id = ifelse(nchar(site.id > 6), substr(site.id, 1, 6),site.id)
-    site.id = ifelse(nchar(site.id < 6), encodeString(site.id, width = 6),site.id)
+    site.id = ifelse(nchar(site.id) > 6, substr(site.id, 1, 6),site.id)
+    site.id = ifelse(nchar(site.id) < 6, encodeString(site.id, width = 6),site.id)
     site.name = ifelse(nchar(site.name) > 52, substr(site.name, 1, 52),site.name)
     site.name = ifelse(nchar(site.name) < 52, encodeString(site.name, width = 52),site.name)
     spp.code = ifelse(nchar(spp.code) > 4, substr(spp.code, 1, 4),spp.code)
@@ -49,17 +49,17 @@ function(crn, fname, header=NULL, append=FALSE)
     state.country = ifelse(nchar(state.country) > 13, substr(state.country, 1, 13),
       state.country)
     state.country = ifelse(nchar(state.country) < 13, encodeString(state.country, width = 13),state.country)
-    spp = ifelse(nchar(spp > 18), substr(spp, 1, 18),spp)
-    spp = ifelse(nchar(spp < 18), encodeString(spp, width = 18),spp)
-    elev = ifelse(nchar(elev > 5), substr(elev, 1, 5),elev)
-    elev = ifelse(nchar(elev < 5), encodeString(elev, width = 5),elev)
+    spp = ifelse(nchar(spp) > 18, substr(spp, 1, 18),spp)
+    spp = ifelse(nchar(spp) < 18, encodeString(spp, width = 18),spp)
+    elev = ifelse(nchar(elev) > 5, substr(elev, 1, 5),elev)
+    elev = ifelse(nchar(elev) < 5, encodeString(elev, width = 5),elev)
     lat.long = ifelse(nchar(long) > 5, paste(lat,long,sep=""),
      paste(lat,long,sep=" "))
-    lat.long = ifelse(nchar(lat.long > 10), substr(lat.long, 1, 10),lat.long)
-    lat.long = ifelse(nchar(lat.long < 10), encodeString(lat.long, width = 10),lat.long)
+    lat.long = ifelse(nchar(lat.long) > 10, substr(lat.long, 1, 10),lat.long)
+    lat.long = ifelse(nchar(lat.long) < 10, encodeString(lat.long, width = 10),lat.long)
     yrs = paste(first.yr,last.yr,sep=" ")
-    yrs = ifelse(nchar(yrs > 9), substr(yrs, 1, 9),yrs)
-    yrs = ifelse(nchar(yrs < 9), encodeString(yrs, width = 9),yrs)
+    yrs = ifelse(nchar(yrs) > 9, substr(yrs, 1, 9),yrs)
+    yrs = ifelse(nchar(yrs) < 9, encodeString(yrs, width = 9),yrs)
     lead.invs = ifelse(nchar(lead.invs) > 63, substr(lead.invs, 1, 63),lead.invs)
     lead.invs = ifelse(nchar(lead.invs) < 63, encodeString(lead.invs, width = 63),lead.invs)
     comp.date = ifelse(nchar(comp.date) > 8, substr(comp.date, 1, 8),comp.date)
@@ -82,30 +82,30 @@ function(crn, fname, header=NULL, append=FALSE)
   # 1-6
   crn.name = colnames(crn)[1]
   crn.width = nchar(crn.name)
-  # Pad to six
-  # If crn.width > 6, truncate
-  crn.name = ifelse(crn.width > 6, substr(crn.name, 1, 6),crn.name)
-  # Pad to nchar 4 (no leading zero)
-  crn.name = ifelse(crn.width < 6,
-    formatC(crn.name, wid = 6, format = "f"),crn.name)
-    
+  ## If crn.width > 6, truncate
+  if(crn.width > 6)
+    crn.name = substr(crn.name, 1, 6)
+  ## Pad to nchar 6 (no leading zero)
+  else if(crn.width < 6)
+    crn.name = formatC(crn.name, wid = 6, format = "f")
 
   dec.str <- character(n.decades)
   for(i in 1:n.decades){
     # 7-10 decade column
     dec = decades[i]
-    n.yrs = table(decades.vec%in%dec)[2]
-    dec.yrs = yrs[decades.vec%in%dec]
+    dec.idx = which(decades.vec%in%dec)
+    n.yrs = length(dec.idx)
+    dec.yrs = yrs[dec.idx]
     # Pad to nchar 4 (no leading zero)
     dec.yrs = formatC(dec.yrs, dig = 0, wid = 4, format = "f")
 
-    dec.rwi = crn[decades.vec%in%dec,1]
+    dec.rwi = crn[dec.idx,1]
     # Pad to nchar 4 (no leading zero)
     dec.rwi = round(dec.rwi,3) * 1000
     dec.rwi = formatC(dec.rwi, dig = 0, wid = 4, format = "f")
 
     # Pad to nchar 3 (no leading zero)
-    dec.samp.depth = crn[decades.vec%in%dec,2]
+    dec.samp.depth = crn[dec.idx,2]
     dec.samp.depth = formatC(dec.samp.depth, dig = 0, wid = 3, format = "f")
     # Pad front end
     if(i == 1){
@@ -127,7 +127,6 @@ function(crn, fname, header=NULL, append=FALSE)
       dec.str[i] = paste(crn.name,dec.yrs[1],dec.rwi[1],dec.samp.depth[1],sep="")
     }
     for(j in 2:n.yrs){
-      j.yr = dec.yrs[j]
       j.rwi = dec.rwi[j]
       j.samp.depth = dec.samp.depth[j]
       dec.str[i] = paste(dec.str[i],j.rwi,j.samp.depth,sep="")
